@@ -43,13 +43,6 @@
 #include "gstaasink.h"
 #include <gst/video/video.h>
 
-/* elementfactory information */
-static const GstElementDetails gst_aasink_details =
-GST_ELEMENT_DETAILS ("ASCII art video sink",
-    "Sink/Video",
-    "An ASCII art videosink",
-    "Wim Taymans <wim.taymans@chello.be>");
-
 /* aasink signals and args */
 enum
 {
@@ -199,7 +192,9 @@ gst_aasink_base_init (gpointer g_class)
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sink_template));
-  gst_element_class_set_details (element_class, &gst_aasink_details);
+  gst_element_class_set_details_simple (element_class, "ASCII art video sink",
+      "Sink/Video",
+      "An ASCII art videosink", "Wim Taymans <wim.taymans@chello.be>");
 }
 
 static void
