@@ -25,6 +25,7 @@
 #include <setjmp.h>
 #include <gst/gst.h>
 #include <gst/video/video.h>
+#include <gst/base/gstadapter.h>
 
 /* this is a hack hack hack to get around jpeglib header bugs... */
 #ifdef HAVE_STDLIB_H
@@ -69,7 +70,9 @@ struct _GstJpegDec {
   GstPad  *sinkpad;
   GstPad  *srcpad;
 
-  GstBuffer *tempbuf;
+  GstAdapter *adapter;
+
+  guint8     *cur_buf;
 
   /* TRUE if each input buffer contains a whole jpeg image */
   gboolean packetized;
@@ -118,6 +121,8 @@ struct _GstJpegDec {
   /* arrays for indirect decoding */
   gboolean idr_width_allocated;
   guchar *idr_y[16],*idr_u[16],*idr_v[16];
+  /* current (parsed) image size */
+  guint    rem_img_len;
 };
 
 struct _GstJpegDecClass {
