@@ -42,6 +42,10 @@
 
 #include <string.h>
 
+#if HAVE_ORC
+#include <orc/orc.h>
+#endif
+
 GST_DEBUG_CATEGORY_STATIC (deinterlace_debug);
 #define GST_CAT_DEFAULT (deinterlace_debug)
 
@@ -618,6 +622,14 @@ gst_deinterlace_reset (GstDeinterlace * self)
   self->fps_n = self->fps_d = 0;
   self->passthrough = FALSE;
 
+  self->reconfigure = FALSE;
+  if (self->new_mode != -1)
+    self->mode = self->new_mode;
+  if (self->new_fields != -1)
+    self->fields = self->new_fields;
+  self->new_mode = -1;
+  self->new_fields = -1;
+
   gst_segment_init (&self->segment, GST_FORMAT_UNDEFINED);
 
   if (self->sink_caps)
@@ -648,14 +660,17 @@ gst_deinterlace_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_MODE:{
-      gint oldmode;
+      gint new_mode;
 
       GST_OBJECT_LOCK (self);
-      oldmode = self->mode;
-      self->mode = g_value_get_enum (value);
-      gst_deinterlace_update_passthrough (self);
-      if (self->mode != oldmode && GST_PAD_CAPS (self->srcpad))
-        gst_deinterlace_setcaps (self->sinkpad, GST_PAD_CAPS (self->sinkpad));
+      new_mode = g_value_get_enum (value);
+      if (self->mode != new_mode && GST_PAD_CAPS (self->srcpad)) {
+        self->reconfigure = TRUE;
+        self->new_mode = new_mode;
+      } else {
+        self->mode = new_mode;
+        gst_deinterlace_update_passthrough (self);
+      }
       GST_OBJECT_UNLOCK (self);
       break;
     }
@@ -663,13 +678,16 @@ gst_deinterlace_set_property (GObject * object, guint prop_id,
       gst_deinterlace_set_method (self, g_value_get_enum (value));
       break;
     case PROP_FIELDS:{
-      gint oldfields;
+      gint new_fields;
 
       GST_OBJECT_LOCK (self);
-      oldfields = self->fields;
-      self->fields = g_value_get_enum (value);
-      if (self->fields != oldfields && GST_PAD_CAPS (self->srcpad))
-        gst_deinterlace_setcaps (self->sinkpad, GST_PAD_CAPS (self->sinkpad));
+      new_fields = g_value_get_enum (value);
+      if (self->fields != new_fields && GST_PAD_CAPS (self->srcpad)) {
+        self->reconfigure = TRUE;
+        self->new_fields = new_fields;
+      } else {
+        self->fields = new_fields;
+      }
       GST_OBJECT_UNLOCK (self);
       break;
     }
@@ -787,15 +805,15 @@ gst_deinterlace_push_history (GstDeinterlace * self, GstBuffer * buffer)
 
   if (field_layout == GST_DEINTERLACE_LAYOUT_TFF) {
     GST_DEBUG_OBJECT (self, "Top field first");
-    field1 = gst_buffer_ref (buffer);
+    field1 = gst_buffer_make_metadata_writable (gst_buffer_ref (buffer));
     field1_flags = PICTURE_INTERLACED_TOP;
-    field2 = gst_buffer_ref (buffer);
+    field2 = gst_buffer_make_metadata_writable (gst_buffer_ref (buffer));
     field2_flags = PICTURE_INTERLACED_BOTTOM;
   } else {
     GST_DEBUG_OBJECT (self, "Bottom field first");
-    field1 = gst_buffer_ref (buffer);
+    field1 = gst_buffer_make_metadata_writable (gst_buffer_ref (buffer));
     field1_flags = PICTURE_INTERLACED_BOTTOM;
-    field2 = gst_buffer_ref (buffer);
+    field2 = gst_buffer_make_metadata_writable (gst_buffer_ref (buffer));
     field2_flags = PICTURE_INTERLACED_TOP;
   }
 
@@ -928,6 +946,22 @@ gst_deinterlace_chain (GstPad * pad, GstBuffer * buf)
   gint cur_field_idx = 0;
   GstBuffer *outbuf;
 
+  GST_OBJECT_LOCK (self);
+  if (self->reconfigure) {
+    if (self->new_fields != -1)
+      self->fields = self->new_fields;
+    if (self->new_mode != -1)
+      self->mode = self->new_mode;
+    self->new_mode = self->new_fields = -1;
+
+    self->reconfigure = FALSE;
+    GST_OBJECT_UNLOCK (self);
+    if (GST_PAD_CAPS (self->srcpad))
+      gst_deinterlace_setcaps (self->sinkpad, GST_PAD_CAPS (self->sinkpad));
+  } else {
+    GST_OBJECT_UNLOCK (self);
+  }
+
   if (self->still_frame_mode || self->passthrough)
     return gst_pad_push (self->srcpad, buf);
 
@@ -1729,7 +1763,9 @@ plugin_init (GstPlugin * plugin)
 {
   GST_DEBUG_CATEGORY_INIT (deinterlace_debug, "deinterlace", 0, "Deinterlacer");
 
-  oil_init ();
+#if HAVE_ORC
+  orc_init ();
+#endif
 
   if (!gst_element_register (plugin, "deinterlace", GST_RANK_NONE,
           GST_TYPE_DEINTERLACE)) {
