@@ -25,10 +25,6 @@
 #include <gst/gst.h>
 #include <gst/video/video.h>
 
-#include <liboil/liboil.h>
-#include <liboil/liboilcpu.h>
-#include <liboil/liboilfunction.h>
-
 #include "gstdeinterlacemethod.h"
 
 G_BEGIN_DECLS
@@ -137,6 +133,10 @@ struct _GstDeinterlace
   GstCaps *src_caps;
 
   GstCaps *request_caps;
+
+  gboolean reconfigure;
+  GstDeinterlaceMode new_mode;
+  GstDeinterlaceFields new_fields;
 };
 
 struct _GstDeinterlaceClass
