@@ -29,13 +29,6 @@
 GST_DEBUG_CATEGORY_STATIC (rtpmp4vdepay_debug);
 #define GST_CAT_DEFAULT (rtpmp4vdepay_debug)
 
-/* elementfactory information */
-static const GstElementDetails gst_rtp_mp4vdepay_details =
-GST_ELEMENT_DETAILS ("RTP MPEG4 video depayloader",
-    "Codec/Depayloader/Network",
-    "Extracts MPEG4 video from RTP packets (RFC 3016)",
-    "Wim Taymans <wim.taymans@gmail.com>");
-
 static GstStaticPadTemplate gst_rtp_mp4v_depay_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -84,7 +77,10 @@ gst_rtp_mp4v_depay_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_mp4v_depay_sink_template));
 
-  gst_element_class_set_details (element_class, &gst_rtp_mp4vdepay_details);
+  gst_element_class_set_details_simple (element_class,
+      "RTP MPEG4 video depayloader", "Codec/Depayloader/Network",
+      "Extracts MPEG4 video from RTP packets (RFC 3016)",
+      "Wim Taymans <wim.taymans@gmail.com>");
 }
 
 static void
@@ -133,14 +129,11 @@ static gboolean
 gst_rtp_mp4v_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
 {
   GstStructure *structure;
-  GstRtpMP4VDepay *rtpmp4vdepay;
   GstCaps *srccaps;
   const gchar *str;
   gint clock_rate;
   gboolean res;
 
-  rtpmp4vdepay = GST_RTP_MP4V_DEPAY (depayload);
-
   structure = gst_caps_get_structure (caps, 0);
 
   if (!gst_structure_get_int (structure, "clock-rate", &clock_rate))
@@ -159,11 +152,10 @@ gst_rtp_mp4v_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
       GstBuffer *buffer;
 
       buffer = gst_value_get_buffer (&v);
-      gst_buffer_ref (buffer);
-      g_value_unset (&v);
-
       gst_caps_set_simple (srccaps,
           "codec_data", GST_TYPE_BUFFER, buffer, NULL);
+      /* caps takes ref */
+      g_value_unset (&v);
     } else {
       g_warning ("cannot convert config to buffer");
     }
