@@ -134,13 +134,6 @@ static GstRtpPtDemuxPad *find_pad_for_pt (GstRtpPtDemux * rtpdemux, guint8 pt);
 
 static guint gst_rtp_pt_demux_signals[LAST_SIGNAL] = { 0 };
 
-static GstElementDetails gst_rtp_pt_demux_details = {
-  "RTP Demux",
-  "Demux/Network/RTP",
-  "Parses codec streams transmitted in the same RTP session",
-  "Kai Vehmanen <kai.vehmanen@nokia.com>"
-};
-
 static void
 gst_rtp_pt_demux_base_init (gpointer g_class)
 {
@@ -151,7 +144,10 @@ gst_rtp_pt_demux_base_init (gpointer g_class)
   gst_element_class_add_pad_template (gstelement_klass,
       gst_static_pad_template_get (&rtp_pt_demux_src_template));
 
-  gst_element_class_set_details (gstelement_klass, &gst_rtp_pt_demux_details);
+  gst_element_class_set_details_simple (gstelement_klass, "RTP Demux",
+      "Demux/Network/RTP",
+      "Parses codec streams transmitted in the same RTP session",
+      "Kai Vehmanen <kai.vehmanen@nokia.com>");
 }
 
 static void
@@ -216,7 +212,7 @@ gst_rtp_pt_demux_class_init (GstRtpPtDemuxClass * klass)
           clear_pt_map), NULL, NULL, g_cclosure_marshal_VOID__VOID,
       G_TYPE_NONE, 0, G_TYPE_NONE);
 
-  gobject_klass->finalize = GST_DEBUG_FUNCPTR (gst_rtp_pt_demux_finalize);
+  gobject_klass->finalize = gst_rtp_pt_demux_finalize;
 
   gstelement_klass->change_state =
       GST_DEBUG_FUNCPTR (gst_rtp_pt_demux_change_state);
@@ -433,7 +429,7 @@ gst_rtp_pt_demux_sink_event (GstPad * pad, GstEvent * event)
   GstRtpPtDemux *rtpdemux;
   gboolean res = FALSE;
 
-  rtpdemux = GST_RTP_PT_DEMUX (GST_PAD_PARENT (pad));
+  rtpdemux = GST_RTP_PT_DEMUX (gst_pad_get_parent (pad));
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_CUSTOM_DOWNSTREAM:
@@ -448,15 +444,20 @@ gst_rtp_pt_demux_sink_event (GstPad * pad, GstEvent * event)
 
         if (rtpdemuxpad)
           res = gst_pad_push_event (rtpdemuxpad->pad, event);
+        else
+          gst_event_unref (event);
 
       } else {
         res = gst_pad_event_default (pad, event);
       }
+      break;
     }
     default:
       res = gst_pad_event_default (pad, event);
       break;
   }
+
+  gst_object_unref (rtpdemux);
   return res;
 }
 
