@@ -22,72 +22,46 @@
  * SECTION:element-udpsrc
  * @see_also: udpsink, multifdsink
  *
- * <refsect2>
- * <para>
  * udpsrc is a network source that reads UDP packets from the network.
  * It can be combined with RTP depayloaders to implement RTP streaming.
- * </para>
- * <title>Examples</title>
- * <para>
- * Here is a simple pipeline to read from the default port and dump the udp packets.
- * <programlisting>
- * gst-launch -v udpsrc ! fakesink dump=1
- * </programlisting>
- * To actually generate udp packets on the default port one can use the
- * udpsink element. When running the following pipeline in another terminal, the
- * above mentioned pipeline should dump data packets to the console.
- * <programlisting>
- * gst-launch -v audiotestsrc ! udpsink
- * </programlisting>
- * </para>
- * <para>
+ *
  * The udpsrc element supports automatic port allocation by setting the
- * "port" property to 0. the following pipeline reads UDP from a free port.
- * <programlisting>
- * gst-launch -v udpsrc port=0 ! fakesink
- * </programlisting>
- * After setting the udpsrc to PAUSED, the allocated port can be obtained by
- * reading the port property.
- * </para>
- * <para>
- * udpsrc can read from multicast groups by setting the multicast_group property
- * to the IP address of the multicast group.
- * </para>
- * <para>
- * Alternatively one can provide a custom socket to udpsrc with the "sockfd" property,
- * udpsrc will then not allocate a socket itself but use the provided one.
- * </para>
- * <para>
- * The "caps" property is mainly used to give a type to the UDP packet so that they
- * can be autoplugged in GStreamer pipelines. This is very usefull for RTP 
- * implementations where the contents of the UDP packets is transfered out-of-bounds
- * using SDP or other means. 
- * </para>
- * <para>
- * The "buffer" property is used to change the default kernel buffer sizes used for
- * receiving packets. The buffer size may be increased for high-volume connections,
- * or may be decreased to limit the possible backlog of incoming data.
- * The system places an absolute limit on these values, on Linux, for example, the
- * default buffer size is typically 50K and can be increased to maximally 100K.
- * </para>
- * <para>
- * The "skip-first-bytes" property is used to strip off an arbitrary number of
- * bytes from the start of the raw udp packet and can be used to strip off
- * proprietary header, for example. 
- * </para>
- * <para>
- * The udpsrc is always a live source. It does however not provide a GstClock, this
- * is left for upstream elements such as an RTP session manager or demuxer (such
- * as an MPEG demuxer). As with all live sources, the captured buffers will have
- * their timestamp set to the current running time of the pipeline.
- * </para>
- * <para>
- * udpsrc implements a GstURIHandler interface that handles udp://host:port type
- * URIs.
- * </para>
- * <para>
- * If the <link linkend="GstUDPSrc--timeout">timeout property</link> is set to a
- * value bigger than 0, udpsrc will generate an element message named
+ * #GstUDPSrc:port property to 0. After setting the udpsrc to PAUSED, the
+ * allocated port can be obtained by reading the port property.
+ *
+ * udpsrc can read from multicast groups by setting the #GstUDPSrc:multicast-group
+ * property to the IP address of the multicast group.
+ *
+ * Alternatively one can provide a custom socket to udpsrc with the #GstUDPSrc:sockfd
+ * property, udpsrc will then not allocate a socket itself but use the provided
+ * one.
+ *
+ * The #GstUDPSrc:caps property is mainly used to give a type to the UDP packet
+ * so that they can be autoplugged in GStreamer pipelines. This is very usefull
+ * for RTP implementations where the contents of the UDP packets is transfered
+ * out-of-bounds using SDP or other means.
+ *
+ * The #GstUDPSrc:buffer property is used to change the default kernel buffer
+ * sizes used for receiving packets. The buffer size may be increased for
+ * high-volume connections, or may be decreased to limit the possible backlog of
+ * incoming data. The system places an absolute limit on these values, on Linux,
+ * for example, the default buffer size is typically 50K and can be increased to
+ * maximally 100K.
+ *
+ * The #GstUDPSrc:skip-first-bytes property is used to strip off an arbitrary
+ * number of bytes from the start of the raw udp packet and can be used to strip
+ * off proprietary header, for example. 
+ *
+ * The udpsrc is always a live source. It does however not provide a #GstClock,
+ * this is left for upstream elements such as an RTP session manager or demuxer
+ * (such as an MPEG demuxer). As with all live sources, the captured buffers
+ * will have their timestamp set to the current running time of the pipeline.
+ *
+ * udpsrc implements a #GstURIHandler interface that handles udp://host:port
+ * type URIs.
+ *
+ * If the #GstUDPSrc:timeout property is set to a value bigger than 0, udpsrc
+ * will generate an element message named
  * <classname>&quot;GstUDPSrcTimeout&quot;</classname>
  * if no data was recieved in the given timeout.
  * The message's structure contains one field:
@@ -105,15 +79,28 @@
  * </para>
  * <para>
  * A custom file descriptor can be configured with the 
- * <link linkend="GstUDPSrc--sockfd">sockfd property</link>. The socket will be
- * closed when setting the element to READY by default. This behaviour can be
- * overriden with the <link linkend="GstUDPSrc--closefd">closefd property</link>,
- * in which case the application is responsible for closing the file descriptor.
- * </para>
- * <para>
- * Last reviewed on 2007-09-20 (0.10.7)
- * </para>
+ * #GstUDPSrc:sockfd property. The socket will be closed when setting the
+ * element to READY by default. This behaviour can be
+ * overriden with the #GstUDPSrc:closefd property, in which case the application
+ * is responsible for closing the file descriptor.
+ *
+ * <refsect2>
+ * <title>Examples</title>
+ * |[
+ * gst-launch -v udpsrc ! fakesink dump=1
+ * ]| A pipeline to read from the default port and dump the udp packets.
+ * To actually generate udp packets on the default port one can use the
+ * udpsink element. When running the following pipeline in another terminal, the
+ * above mentioned pipeline should dump data packets to the console.
+ * |[
+ * gst-launch -v audiotestsrc ! udpsink
+ * ]|
+ * |[
+ * gst-launch -v udpsrc port=0 ! fakesink
+ * ]| read udp packets from a free port.
  * </refsect2>
+ *
+ * Last reviewed on 2007-09-20 (0.10.7)
  */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -146,15 +133,9 @@ static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS_ANY);
 
-static const GstElementDetails gst_udpsrc_details =
-GST_ELEMENT_DETAILS ("UDP packet receiver",
-    "Source/Network",
-    "Receive data over the network via UDP",
-    "Wim Taymans <wim@fluendo.com>\n"
-    "Thijs Vermeir <thijs.vermeir@barco.com>");
-
 #define UDP_DEFAULT_PORT                4951
 #define UDP_DEFAULT_MULTICAST_GROUP     "0.0.0.0"
+#define UDP_DEFAULT_MULTICAST_IFACE     NULL
 #define UDP_DEFAULT_URI                 "udp://"UDP_DEFAULT_MULTICAST_GROUP":"G_STRINGIFY(UDP_DEFAULT_PORT)
 #define UDP_DEFAULT_CAPS                NULL
 #define UDP_DEFAULT_SOCKFD              -1
@@ -164,6 +145,7 @@ GST_ELEMENT_DETAILS ("UDP packet receiver",
 #define UDP_DEFAULT_CLOSEFD            TRUE
 #define UDP_DEFAULT_SOCK                -1
 #define UDP_DEFAULT_AUTO_MULTICAST     TRUE
+#define UDP_DEFAULT_REUSE              TRUE
 
 enum
 {
@@ -171,6 +153,7 @@ enum
 
   PROP_PORT,
   PROP_MULTICAST_GROUP,
+  PROP_MULTICAST_IFACE,
   PROP_URI,
   PROP_CAPS,
   PROP_SOCKFD,
@@ -180,6 +163,7 @@ enum
   PROP_CLOSEFD,
   PROP_SOCK,
   PROP_AUTO_MULTICAST,
+  PROP_REUSE,
 
   PROP_LAST
 };
@@ -240,7 +224,11 @@ gst_udpsrc_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_template));
 
-  gst_element_class_set_details (element_class, &gst_udpsrc_details);
+  gst_element_class_set_details_simple (element_class, "UDP packet receiver",
+      "Source/Network",
+      "Receive data over the network via UDP",
+      "Wim Taymans <wim@fluendo.com>, "
+      "Thijs Vermeir <thijs.vermeir@barco.com>");
 }
 
 static void
@@ -263,9 +251,13 @@ gst_udpsrc_class_init (GstUDPSrcClass * klass)
           "The port to receive the packets from, 0=allocate", 0, G_MAXUINT16,
           UDP_DEFAULT_PORT, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_MULTICAST_GROUP,
-      g_param_spec_string ("multicast_group", "Multicast Group",
+      g_param_spec_string ("multicast-group", "Multicast Group",
           "The Address of multicast group to join", UDP_DEFAULT_MULTICAST_GROUP,
           G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_MULTICAST_IFACE,
+      g_param_spec_string ("multicast-iface", "Multicast Interface",
+          "The network interface on which to join the multicast group",
+          UDP_DEFAULT_MULTICAST_IFACE, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_URI,
       g_param_spec_string ("uri", "URI",
           "URI in the form of udp://multicast_group:port", UDP_DEFAULT_URI,
@@ -301,6 +293,9 @@ gst_udpsrc_class_init (GstUDPSrcClass * klass)
       g_param_spec_boolean ("auto-multicast", "Auto Multicast",
           "Automatically join/leave multicast groups",
           UDP_DEFAULT_AUTO_MULTICAST, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_REUSE,
+      g_param_spec_boolean ("reuse", "Reuse", "Enable reuse of the port",
+          UDP_DEFAULT_REUSE, G_PARAM_READWRITE));
 
   gstbasesrc_class->start = gst_udpsrc_start;
   gstbasesrc_class->stop = gst_udpsrc_stop;
@@ -316,10 +311,11 @@ gst_udpsrc_init (GstUDPSrc * udpsrc, GstUDPSrcClass * g_class)
 {
   WSA_STARTUP (udpsrc);
 
-  udpsrc->port = UDP_DEFAULT_PORT;
+  gst_udp_uri_init (&udpsrc->uri, UDP_DEFAULT_MULTICAST_GROUP,
+      UDP_DEFAULT_PORT);
+
   udpsrc->sockfd = UDP_DEFAULT_SOCKFD;
-  udpsrc->multi_group = g_strdup (UDP_DEFAULT_MULTICAST_GROUP);
-  udpsrc->uri = g_strdup (UDP_DEFAULT_URI);
+  udpsrc->multi_iface = g_strdup (UDP_DEFAULT_MULTICAST_IFACE);
   udpsrc->buffer_size = UDP_DEFAULT_BUFFER_SIZE;
   udpsrc->timeout = UDP_DEFAULT_TIMEOUT;
   udpsrc->skip_first_bytes = UDP_DEFAULT_SKIP_FIRST_BYTES;
@@ -327,6 +323,7 @@ gst_udpsrc_init (GstUDPSrc * udpsrc, GstUDPSrcClass * g_class)
   udpsrc->externalfd = (udpsrc->sockfd != -1);
   udpsrc->auto_multicast = UDP_DEFAULT_AUTO_MULTICAST;
   udpsrc->sock.fd = UDP_DEFAULT_SOCK;
+  udpsrc->reuse = UDP_DEFAULT_REUSE;
 
   /* configure basesrc to be a live source */
   gst_base_src_set_live (GST_BASE_SRC (udpsrc), TRUE);
@@ -346,8 +343,11 @@ gst_udpsrc_finalize (GObject * object)
 
   if (udpsrc->caps)
     gst_caps_unref (udpsrc->caps);
-  g_free (udpsrc->multi_group);
-  g_free (udpsrc->uri);
+
+  g_free (udpsrc->multi_iface);
+
+  gst_udp_uri_free (&udpsrc->uri);
+  g_free (udpsrc->uristr);
 
   if (udpsrc->sockfd >= 0 && udpsrc->closefd)
     CLOSE_SOCKET (udpsrc->sockfd);
@@ -370,13 +370,50 @@ gst_udpsrc_getcaps (GstBaseSrc * src)
     return gst_caps_new_any ();
 }
 
+/* read a message from the error queue */
+static void
+clear_error (GstUDPSrc * udpsrc)
+{
+#if defined (MSG_ERRQUEUE)
+  struct msghdr cmsg;
+  char cbuf[128];
+  char msgbuf[CMSG_SPACE (128)];
+  struct iovec iov;
+
+  /* Flush ERRORS from fd so next poll will not return at once */
+  /* No need for address : We look for local error */
+  cmsg.msg_name = NULL;
+  cmsg.msg_namelen = 0;
+
+  /* IOV */
+  memset (&cbuf, 0, sizeof (cbuf));
+  iov.iov_base = cbuf;
+  iov.iov_len = sizeof (cbuf);
+  cmsg.msg_iov = &iov;
+  cmsg.msg_iovlen = 1;
+
+  /* msg_control */
+  memset (&msgbuf, 0, sizeof (msgbuf));
+  cmsg.msg_control = &msgbuf;
+  cmsg.msg_controllen = sizeof (msgbuf);
+
+  recvmsg (udpsrc->sock.fd, &cmsg, MSG_ERRQUEUE);
+#endif
+}
+
 static GstFlowReturn
 gst_udpsrc_create (GstPushSrc * psrc, GstBuffer ** buf)
 {
   GstUDPSrc *udpsrc;
   GstNetBuffer *outbuf;
-  struct sockaddr_storage tmpaddr;
-  socklen_t len;
+  union gst_sockaddr
+  {
+    struct sockaddr sa;
+    struct sockaddr_in sa_in;
+    struct sockaddr_in6 sa_in6;
+    struct sockaddr_storage sa_stor;
+  } sa;
+  socklen_t slen;
   guint8 *pktdata;
   gint pktsize;
 #ifdef G_OS_UNIX
@@ -447,8 +484,10 @@ retry:
    * woken up by activity on the socket but it was not a read. We know someone
    * will also do something with the socket so that we don't go into an infinite
    * loop in the select(). */
-  if (G_UNLIKELY (!readsize))
+  if (G_UNLIKELY (!readsize)) {
+    clear_error (udpsrc);
     goto retry;
+  }
 
 no_select:
   GST_LOG_OBJECT (udpsrc, "ioctl says %d bytes available", (int) readsize);
@@ -457,9 +496,13 @@ no_select:
   pktsize = readsize;
 
   while (TRUE) {
-    len = sizeof (struct sockaddr);
-    ret = recvfrom (udpsrc->sock.fd, pktdata, pktsize,
-        0, (struct sockaddr *) &tmpaddr, &len);
+    slen = sizeof (sa);
+#ifdef G_OS_WIN32
+    ret = recvfrom (udpsrc->sock.fd, (char *) pktdata, pktsize, 0, &sa.sa,
+        &slen);
+#else
+    ret = recvfrom (udpsrc->sock.fd, pktdata, pktsize, 0, &sa.sa, &slen);
+#endif
     if (G_UNLIKELY (ret < 0)) {
 #ifdef G_OS_WIN32
       /* WSAECONNRESET for a UDP socket means that a packet sent with udpsink
@@ -495,22 +538,19 @@ no_select:
   GST_BUFFER_DATA (outbuf) = pktdata;
   GST_BUFFER_SIZE (outbuf) = ret;
 
-  switch (tmpaddr.ss_family) {
+  switch (sa.sa.sa_family) {
     case AF_INET:
     {
-      gst_netaddress_set_ip4_address (&outbuf->from,
-          ((struct sockaddr_in *) &tmpaddr)->sin_addr.s_addr,
-          ((struct sockaddr_in *) &tmpaddr)->sin_port);
+      gst_netaddress_set_ip4_address (&outbuf->from, sa.sa_in.sin_addr.s_addr,
+          sa.sa_in.sin_port);
     }
       break;
     case AF_INET6:
     {
       guint8 ip6[16];
 
-      memcpy (ip6, &((struct sockaddr_in6 *) &tmpaddr)->sin6_addr,
-          sizeof (ip6));
-      gst_netaddress_set_ip6_address (&outbuf->from, ip6,
-          ((struct sockaddr_in *) &tmpaddr)->sin_port);
+      memcpy (ip6, &sa.sa_in6.sin6_addr, sizeof (ip6));
+      gst_netaddress_set_ip6_address (&outbuf->from, ip6, sa.sa_in6.sin6_port);
     }
       break;
     default:
@@ -565,54 +605,22 @@ skip_error:
   }
 }
 
-/* Call this function when multicastgroup and/or port are updated */
-
-static void
-gst_udpsrc_update_uri (GstUDPSrc * src)
-{
-  g_free (src->uri);
-  src->uri = g_strdup_printf ("udp://%s:%d", src->multi_group, src->port);
-
-  GST_DEBUG_OBJECT (src, "updated uri to %s", src->uri);
-}
-
 static gboolean
 gst_udpsrc_set_uri (GstUDPSrc * src, const gchar * uri)
 {
-  gchar *protocol;
-  gchar *location;
-  gchar *colptr;
+  if (gst_udp_parse_uri (uri, &src->uri) < 0)
+    goto wrong_uri;
 
-  protocol = gst_uri_get_protocol (uri);
-  if (strcmp (protocol, "udp") != 0)
-    goto wrong_protocol;
-  g_free (protocol);
-
-  location = gst_uri_get_location (uri);
-  if (!location)
-    return FALSE;
-  colptr = strrchr (location, ':');
-  if (colptr != NULL) {
-    g_free (src->multi_group);
-    src->multi_group = g_strndup (location, colptr - location);
-    src->port = atoi (colptr + 1);
-  } else {
-    g_free (src->multi_group);
-    src->multi_group = g_strdup (location);
-    src->port = UDP_DEFAULT_PORT;
-  }
-  g_free (location);
-
-  gst_udpsrc_update_uri (src);
+  if (src->uri.port == -1)
+    src->uri.port = UDP_DEFAULT_PORT;
 
   return TRUE;
 
   /* ERRORS */
-wrong_protocol:
+wrong_uri:
   {
-    g_free (protocol);
     GST_ELEMENT_ERROR (src, RESOURCE, READ, (NULL),
-        ("error parsing uri %s: wrong protocol (%s != udp)", uri, protocol));
+        ("error parsing uri %s", uri));
     return FALSE;
   }
 }
@@ -628,17 +636,25 @@ gst_udpsrc_set_property (GObject * object, guint prop_id, const GValue * value,
       udpsrc->buffer_size = g_value_get_int (value);
       break;
     case PROP_PORT:
-      udpsrc->port = g_value_get_int (value);
-      gst_udpsrc_update_uri (udpsrc);
+      gst_udp_uri_update (&udpsrc->uri, NULL, g_value_get_int (value));
       break;
     case PROP_MULTICAST_GROUP:
-      g_free (udpsrc->multi_group);
+    {
+      const gchar *group;
+
+      if ((group = g_value_get_string (value)))
+        gst_udp_uri_update (&udpsrc->uri, group, -1);
+      else
+        gst_udp_uri_update (&udpsrc->uri, UDP_DEFAULT_MULTICAST_GROUP, -1);
+      break;
+    }
+    case PROP_MULTICAST_IFACE:
+      g_free (udpsrc->multi_iface);
 
       if (g_value_get_string (value) == NULL)
-        udpsrc->multi_group = g_strdup (UDP_DEFAULT_MULTICAST_GROUP);
+        udpsrc->multi_iface = g_strdup (UDP_DEFAULT_MULTICAST_IFACE);
       else
-        udpsrc->multi_group = g_value_dup_string (value);
-      gst_udpsrc_update_uri (udpsrc);
+        udpsrc->multi_iface = g_value_dup_string (value);
       break;
     case PROP_URI:
       gst_udpsrc_set_uri (udpsrc, g_value_get_string (value));
@@ -683,6 +699,9 @@ gst_udpsrc_set_property (GObject * object, guint prop_id, const GValue * value,
     case PROP_AUTO_MULTICAST:
       udpsrc->auto_multicast = g_value_get_boolean (value);
       break;
+    case PROP_REUSE:
+      udpsrc->reuse = g_value_get_boolean (value);
+      break;
     default:
       break;
   }
@@ -699,13 +718,16 @@ gst_udpsrc_get_property (GObject * object, guint prop_id, GValue * value,
       g_value_set_int (value, udpsrc->buffer_size);
       break;
     case PROP_PORT:
-      g_value_set_int (value, udpsrc->port);
+      g_value_set_int (value, udpsrc->uri.port);
       break;
     case PROP_MULTICAST_GROUP:
-      g_value_set_string (value, udpsrc->multi_group);
+      g_value_set_string (value, udpsrc->uri.host);
+      break;
+    case PROP_MULTICAST_IFACE:
+      g_value_set_string (value, udpsrc->multi_iface);
       break;
     case PROP_URI:
-      g_value_set_string (value, udpsrc->uri);
+      g_value_take_string (value, gst_udp_uri_string (&udpsrc->uri));
       break;
     case PROP_CAPS:
       gst_value_set_caps (value, udpsrc->caps);
@@ -728,6 +750,9 @@ gst_udpsrc_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_AUTO_MULTICAST:
       g_value_set_boolean (value, udpsrc->auto_multicast);
       break;
+    case PROP_REUSE:
+      g_value_set_boolean (value, udpsrc->reuse);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -739,21 +764,26 @@ static gboolean
 gst_udpsrc_start (GstBaseSrc * bsrc)
 {
   guint bc_val;
+  guint err_val;
   gint reuse;
   int port;
   GstUDPSrc *src;
   gint ret;
   int rcvsize;
+  struct sockaddr_storage bind_address;
+#ifdef G_OS_WIN32
+  gint len;
+#else
   guint len;
-
+#endif
   src = GST_UDPSRC (bsrc);
 
   if (src->sockfd == -1) {
     /* need to allocate a socket */
-    GST_DEBUG_OBJECT (src, "allocating socket for %s:%d", src->multi_group,
-        src->port);
+    GST_DEBUG_OBJECT (src, "allocating socket for %s:%d", src->uri.host,
+        src->uri.port);
     if ((ret =
-            gst_udp_get_addr (src->multi_group, src->port, &src->myaddr)) < 0)
+            gst_udp_get_addr (src->uri.host, src->uri.port, &src->myaddr)) < 0)
       goto getaddrinfo_error;
 
     if ((ret = socket (src->myaddr.ss_family, SOCK_DGRAM, IPPROTO_UDP)) < 0)
@@ -762,34 +792,47 @@ gst_udpsrc_start (GstBaseSrc * bsrc)
     src->sock.fd = ret;
     src->externalfd = FALSE;
 
-    reuse = 1;
+    GST_DEBUG_OBJECT (src, "got socket %d", src->sock.fd);
+
+    GST_DEBUG_OBJECT (src, "setting reuse %d", src->reuse);
+    reuse = src->reuse ? 1 : 0;
     if ((ret =
             setsockopt (src->sock.fd, SOL_SOCKET, SO_REUSEADDR, &reuse,
                 sizeof (reuse))) < 0)
       goto setsockopt_error;
 
-    GST_DEBUG_OBJECT (src, "binding on port %d", src->port);
+    GST_DEBUG_OBJECT (src, "binding on port %d", src->uri.port);
 
-    /* Mac OS is picky about the size for the bind so we switch on the family */
-    switch (src->myaddr.ss_family) {
-      case AF_INET:
-        len = sizeof (struct sockaddr_in);
-        break;
-      case AF_INET6:
-        len = sizeof (struct sockaddr_in6);
-        break;
-      default:
-        /* don't know, Screw MacOS and use the full length */
-        len = sizeof (src->myaddr);
-        break;
+    /* Take a temporary copy of the address in case we need to fix it for bind */
+    memcpy (&bind_address, &src->myaddr, sizeof (struct sockaddr_storage));
+
+#ifdef G_OS_WIN32
+    /* Windows does not allow binding to a multicast group so fix source address */
+    if (gst_udp_is_multicast (&src->myaddr)) {
+      switch (((struct sockaddr *) &bind_address)->sa_family) {
+        case AF_INET:
+          ((struct sockaddr_in *) &bind_address)->sin_addr.s_addr =
+              htonl (INADDR_ANY);
+          break;
+        case AF_INET6:
+          ((struct sockaddr_in6 *) &bind_address)->sin6_addr = in6addr_any;
+          break;
+        default:
+          break;
+      }
     }
-    if ((ret = bind (src->sock.fd, (struct sockaddr *) &src->myaddr, len)) < 0)
+#endif
+
+    len = gst_udp_get_sockaddr_length (&bind_address);
+    if ((ret = bind (src->sock.fd, (struct sockaddr *) &bind_address, len)) < 0)
       goto bind_error;
 
-    len = sizeof (src->myaddr);
-    if ((ret = getsockname (src->sock.fd, (struct sockaddr *) &src->myaddr,
-                &len)) < 0)
-      goto getsockname_error;
+    if (!gst_udp_is_multicast (&src->myaddr)) {
+      len = sizeof (src->myaddr);
+      if ((ret = getsockname (src->sock.fd, (struct sockaddr *) &src->myaddr,
+                  &len)) < 0)
+        goto getsockname_error;
+    }
   } else {
     GST_DEBUG_OBJECT (src, "using provided socket %d", src->sockfd);
     /* we use the configured socket, try to get some info about it */
@@ -839,9 +882,20 @@ gst_udpsrc_start (GstBaseSrc * bsrc)
             g_strerror (errno), errno));
   }
 
+  /* Accept ERRQUEUE to get and flush icmp errors */
+  err_val = 1;
+#if defined (IP_RECVERR)
+  if ((ret = setsockopt (src->sock.fd, IPPROTO_IP, IP_RECVERR, &err_val,
+              sizeof (err_val))) < 0) {
+    GST_ELEMENT_WARNING (src, RESOURCE, SETTINGS, (NULL),
+        ("could not configure socket for IP_RECVERR %d: %s (%d)", ret,
+            g_strerror (errno), errno));
+  }
+#endif
+
   if (src->auto_multicast && gst_udp_is_multicast (&src->myaddr)) {
-    GST_DEBUG_OBJECT (src, "joining multicast group %s", src->multi_group);
-    ret = gst_udp_join_group (src->sock.fd, &src->myaddr);
+    GST_DEBUG_OBJECT (src, "joining multicast group %s", src->uri.host);
+    ret = gst_udp_join_group (src->sock.fd, &src->myaddr, src->multi_iface);
     if (ret < 0)
       goto membership;
   }
@@ -850,8 +904,8 @@ gst_udpsrc_start (GstBaseSrc * bsrc)
    * follows ss_family on both */
   port = g_ntohs (((struct sockaddr_in *) &src->myaddr)->sin_port);
   GST_DEBUG_OBJECT (src, "bound, on port %d", port);
-  if (port != src->port) {
-    src->port = port;
+  if (port != src->uri.port) {
+    src->uri.port = port;
     GST_DEBUG_OBJECT (src, "notifying port %d", port);
     g_object_notify (G_OBJECT (src), "port");
   }
@@ -868,7 +922,7 @@ gst_udpsrc_start (GstBaseSrc * bsrc)
 getaddrinfo_error:
   {
     GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
-        ("getaddrinfo failed %d: %s (%d)", ret, g_strerror (errno), errno));
+        ("getaddrinfo failed: %s (%d)", gai_strerror (ret), ret));
     return FALSE;
   }
 no_socket:
@@ -952,7 +1006,7 @@ gst_udpsrc_stop (GstBaseSrc * bsrc)
 
   if (src->sock.fd >= 0) {
     if (src->auto_multicast && gst_udp_is_multicast (&src->myaddr)) {
-      GST_DEBUG_OBJECT (src, "leaving multicast group %s", src->multi_group);
+      GST_DEBUG_OBJECT (src, "leaving multicast group %s", src->uri.host);
       gst_udp_leave_group (src->sock.fd, &src->myaddr);
     }
     CLOSE_IF_REQUESTED (src);
@@ -977,7 +1031,7 @@ gst_udpsrc_uri_get_type (void)
 static gchar **
 gst_udpsrc_uri_get_protocols (void)
 {
-  static gchar *protocols[] = { "udp", NULL };
+  static gchar *protocols[] = { (char *) "udp", NULL };
 
   return protocols;
 }
@@ -987,7 +1041,10 @@ gst_udpsrc_uri_get_uri (GstURIHandler * handler)
 {
   GstUDPSrc *src = GST_UDPSRC (handler);
 
-  return src->uri;
+  g_free (src->uristr);
+  src->uristr = gst_udp_uri_string (&src->uri);
+
+  return src->uristr;
 }
 
 static gboolean
