@@ -36,7 +36,7 @@
  * <refsect2>
  * <title>Sample pipelines</title>
  * |[
- * gst-launch videotestsrc pattern=1 ! video/x-raw-yuv, framerate=\(fraction\)10/1, width=100, height=100 ! videobox border-alpha=0 alpha=0.5 top=-70 bottom=-70 right=-220 ! videomixer name=mix ! ffmpegcolorspace ! xvimagesink videotestsrc ! video/x-raw-yuv, framerate=\(fraction\)5/1, width=320, height=240 ! alpha alpha=0.7 ! mix.
+ * gst-launch videotestsrc pattern=1 ! video/x-raw-yuv,format=\(fourcc\)AYUV, framerate=\(fraction\)10/1, width=100, height=100 ! videobox border-alpha=0 alpha=0.5 top=-70 bottom=-70 right=-220 ! videomixer name=mix ! ffmpegcolorspace ! xvimagesink videotestsrc ! video/x-raw-yuv, format=\(fourcc\)AYUV, framerate=\(fraction\)5/1, width=320, height=240 ! alpha alpha=0.7 ! mix.
  * ]| A pipeline to demonstrate videomixer used together with videobox.
  * This should show a 320x240 pixels video test source with some transparency
  * showing the background checker pattern. Another video test source with just
@@ -77,7 +77,8 @@
 
 #include "videomixer.h"
 
-GST_DEBUG_CATEGORY (gst_videomixer_debug);
+GST_DEBUG_CATEGORY_STATIC (gst_videomixer_debug);
+#define GST_CAT_DEFAULT gst_videomixer_debug
 
 #define GST_VIDEO_MIXER_GET_STATE_LOCK(mix) \
   (GST_VIDEO_MIXER(mix)->state_lock)
@@ -1751,8 +1752,10 @@ gst_videomixer_sink_event (GstPad * pad, GstEvent * event)
       vpad->queued = 0;
       break;
     case GST_EVENT_NEWSEGMENT:
-      videomixer->sendseg = TRUE;
-      gst_videomixer_reset_qos (videomixer);
+      if (!videomixer->master || vpad == videomixer->master) {
+        videomixer->sendseg = TRUE;
+        gst_videomixer_reset_qos (videomixer);
+      }
       break;
     default:
       break;
