@@ -24,18 +24,14 @@
 /**
  * SECTION:element-sunaudiosrc
  *
- * <refsect2>
- * <para>
  * sunaudiosrc is an audio source designed to work with the Sun Audio
  * interface available in Solaris.
- * </para>
+ *
+ * <refsect2>
  * <title>Example launch line</title>
- * <para>
- * <programlisting>
- * 
- * gst-launch sunaudiosrc ! filesink location=outfile 
- * </programlisting>
- * </para>
+ * |[
+ * gst-launch sunaudiosrc ! wavenc ! filesink location=audio.wav
+ * ]|
  * </refsect2>
  */
 
@@ -54,12 +50,6 @@
 GST_DEBUG_CATEGORY_EXTERN (sunaudio_debug);
 #define GST_CAT_DEFAULT sunaudio_debug
 
-static GstElementDetails plugin_details =
-GST_ELEMENT_DETAILS ("Sun Audio Source",
-    "Source/Audio",
-    "Audio source for Sun Audio devices",
-    "Brian Cameron <brian.cameron@sun.com>");
-
 static void gst_sunaudiosrc_base_init (gpointer g_class);
 static void gst_sunaudiosrc_class_init (GstSunAudioSrcClass * klass);
 static void gst_sunaudiosrc_init (GstSunAudioSrc * sunaudiosrc,
@@ -120,7 +110,10 @@ gst_sunaudiosrc_base_init (gpointer g_class)
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_sunaudiosrc_factory));
-  gst_element_class_set_details (element_class, &plugin_details);
+  gst_element_class_set_details_simple (element_class, "Sun Audio Source",
+      "Source/Audio",
+      "Audio source for Sun Audio devices",
+      "Brian Cameron <brian.cameron@sun.com>");
 }
 
 static void
@@ -138,11 +131,9 @@ gst_sunaudiosrc_class_init (GstSunAudioSrcClass * klass)
   gstbaseaudiosrc_class = (GstBaseAudioSrcClass *) klass;
   gstaudiosrc_class = (GstAudioSrcClass *) klass;
 
-  gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_sunaudiosrc_dispose);
-  gobject_class->get_property =
-      GST_DEBUG_FUNCPTR (gst_sunaudiosrc_get_property);
-  gobject_class->set_property =
-      GST_DEBUG_FUNCPTR (gst_sunaudiosrc_set_property);
+  gobject_class->dispose = gst_sunaudiosrc_dispose;
+  gobject_class->get_property = gst_sunaudiosrc_get_property;
+  gobject_class->set_property = gst_sunaudiosrc_set_property;
 
   gstbasesrc_class->get_caps = GST_DEBUG_FUNCPTR (gst_sunaudiosrc_getcaps);
 
@@ -316,8 +307,8 @@ gst_sunaudiosrc_prepare (GstAudioSrc * asrc, GstRingBufferSpec * spec)
   GstSunAudioSrc *sunaudiosrc = GST_SUNAUDIO_SRC (asrc);
   audio_info_t ainfo;
   int ret;
-  int ctrl_fd = -1;
-  int ports;
+  GstSunAudioMixerCtrl *mixer;
+  struct audio_info audioinfo;
 
   ret = ioctl (sunaudiosrc->fd, AUDIO_GETINFO, &ainfo);
   if (ret == -1) {
@@ -337,8 +328,7 @@ gst_sunaudiosrc_prepare (GstAudioSrc * asrc, GstRingBufferSpec * spec)
   ainfo.record.encoding = AUDIO_ENCODING_LINEAR;
   ainfo.record.buffer_size = spec->buffer_time;
 
-  GstSunAudioMixerCtrl *mixer = sunaudiosrc->mixer;
-  struct audio_info audioinfo;
+  mixer = sunaudiosrc->mixer;
 
   if (ioctl (mixer->mixer_fd, AUDIO_GETINFO, &audioinfo) < 0) {
     g_warning ("Error getting audio device volume");
